#!/usr/bin/python
# -*- coding: UTF-8 -*-
from dli.column import Column
from dli.row import Row

class ResultSetIterator(object):
    def __init__(self, result_set):
        self._result_set = result_set
        self._index = 0

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def next(self):
        try:
            row = self._result_set.get_row(self._index)
        except IndexError:
            raise StopIteration()

        self._index += 1
        return row


class ResultSet(object):
    def __init__(self, job, rows=None, schema=None):
        self._job = job
        self._rows = rows
        self._schema = schema
        self._row_idx = 0
        self._row_count = 0
        self._current_page = 1

        if rows:
            self._row_count = len(rows)
        else:
            self._rows = []
            self._get_result()

    def __iter__(self):
        return ResultSetIterator(self)

    def get_row(self, index):
        if index >= len(self._rows) and index < self.row_count:
            self._get_result()

        return self._rows[index]

    def _get_result(self):
        job_result = self._job.get_result_page(self._current_page)

        self._row_count = job_result.row_count

        if self._schema is None:
            self._schema = []
            for col in job_result.schema:
                for col_name, col_type in col.items():
                    self._schema.append(Column(col_name, col_type))

        for row in job_result.rows:
            self._rows.append(Row(self._schema, row))

        self._current_page += 1

    @property
    def row_count(self):
        return self._row_count

